#!/bin/sh
`dirname $0`/../bin/kc.sh \
    --config-file=`dirname $0`/../conf/keycloak.conf \
    start \
    --proxy-headers xforwarded \
    --hostname-strict=false \
    --http-enabled=true \
    --http-relative-path=/auth \
    --log=console,file \
    --log-level=INFO
